import logging
import numpy as np
from datetime import timedelta
import sys


def flag_length_outliers(df, columnname):
    """Return index of records with length-outliers above 3 standard deviations from the median."""
    return df[columnname] > (np.median(df[columnname]) + 3 * np.std(df[columnname]))


def phred_to_percent(phred):
    return 100 * (1 - 10 ** (phred / -10))


def non_filtered_reads(df):
    return len(df[df["length_filter"]])


def filter_and_transform_data(df, settings):
    '''
    Perform filtering on the data based on arguments set on commandline
    - use aligned length or sequenced length (bam mode only)
    - hide outliers from length plots*
    - hide reads longer than maxlength or shorter than minlength from length plots*
    - filter reads with a quality below minqual
    - use log10 scaled reads rather than normal
    - use empirical percent accuracy rather than phred score quality
    - downsample reads to args.downsample

    - always: drop reads which are basecaller artefacts
              judged by length below 20 and quality above 30

    * using a boolean column length_filter
    '''
    df["length_filter"] = True
    settings["filtered"] = False

    if settings.get("alength") and settings.get("bam"):
        settings["lengths_pointer"] = "aligned_lengths"
        logging.info("Using aligned read lengths for plotting.")
    else:
        settings["lengths_pointer"] = "lengths"
        logging.info("Using sequenced read lengths for plotting.")

    if settings.get("drop_outliers"):
        num_reads_prior = non_filtered_reads(df)
        df.loc[flag_length_outliers(df, settings["lengths_pointer"]), "length_filter"] = False
        num_reads_post = non_filtered_reads(df)
        logging.info("Hidding {} length outliers in length plots.".format(
            str(num_reads_prior - num_reads_post)))

    if settings.get("maxlength"):
        num_reads_prior = non_filtered_reads(df)
        df.loc[df[settings["lengths_pointer"]] > settings["maxlength"], "length_filter"] = False
        num_reads_post = non_filtered_reads(df)
        logging.info("Hidding {} reads longer than {}bp in length plots.".format(
            str(num_reads_prior - num_reads_post),
            str(settings["maxlength"])))

    if settings.get("minlength"):
        num_reads_prior = non_filtered_reads(df)
        df.loc[df[settings["lengths_pointer"]] < settings["minlength"], "length_filter"] = False
        num_reads_post = non_filtered_reads(df)
        logging.info("Hidding {} reads shorter than {}bp in length plots.".format(
            str(num_reads_prior - num_reads_post),
            str(settings["minlength"])))

    if settings.get("minqual"):
        if "quals" in df:
            num_reads_prior = non_filtered_reads(df)
            df = df.loc[df["quals"] > settings["minqual"]].copy()
            num_reads_post = non_filtered_reads(df)
            logging.info("Removing {} reads with quality below Q{}.".format(
                str(num_reads_prior - num_reads_post),
                str(settings["minqual"])))
            settings["filtered"] = True
        else:
            sys.stderr.write("--minqual is ignored since no quality information in the data.")
            logging.info("--minqual is ignored since no quality information in the data.")

    if settings.get("loglength"):
        df["log_" + settings["lengths_pointer"]] = np.log10(df[settings["lengths_pointer"]])
        settings["lengths_pointer"] = "log_" + settings["lengths_pointer"]
        logging.info("Using log10 scaled read lengths.")
        settings["logBool"] = True
    else:
        settings["logBool"] = False

    if settings.get("runtime_until"):
        if "start_time" in df:
            num_reads_prior = non_filtered_reads(df)
            df = df[df.start_time < timedelta(hours=settings["runtime_until"])]
            num_reads_post = non_filtered_reads(df)
            logging.info("Removing {} reads generated after {} hours in the run.".format(
                str(num_reads_prior - num_reads_post),
                str(settings["runtime_until"])))
            settings["filtered"] = True
        else:
            sys.stderr.write("--runtime_until is ignored since no time information in the data.")
            logging.info("--runtime_until is ignored since no time information in the data.")

    if "quals" in df:
        num_reads_prior = len(df)
        df = df.loc[-((df["lengths"] < 20) & (df["quals"] > 30))].copy()
        num_reads_post = len(df)
        if num_reads_prior - num_reads_post > 0:
            logging.info(
                "Removed {} artefactual reads with very short length and very high quality."
                .format(num_reads_prior - num_reads_post))
            settings["filtered"] = True

    if settings.get("downsample"):
        new_size = min(settings["downsample"], len(df))
        logging.info("Downsampling the dataset from {} to {} reads".format(
            len(df), new_size))
        df = df.sample(new_size)
        settings["filtered"] = True

    if settings.get("percentqual"):
        df["quals"] = df["quals"].apply(phred_to_percent)
        logging.info("Converting quality scores to theoretical percent identities.")

    return(df, settings)



import pandas as pd
from argparse import ArgumentParser


def main():
    parser = ArgumentParser(
        description="Add barcode_arrangement to sequencing_summary.txt using barcoding_summary.txt")
    parser.add_argument("sequencing_summary",
                        help="sequencing_summary.txt from guppy, can be compressed.")
    parser.add_argument("barcoding_summary",
                        help="barcoding_summary.txt from guppy, can be compressed.")
    args = parser.parse_args()
    bc = pd.read_csv(args.barcoding_summary, sep="\t", usecols=['read_id', 'barcode_arrangement'])
    df = pd.read_csv(args.sequencing_summary, sep="\t")
    df.join(bc.set_index('read_id'), on='read_id') \
      .to_csv(path_or_buf="barcoded_sequencing_summary.txt.gz",
              sep="\t",
              compression="gzip",
              index=False)


if __name__ == '__main__':
    main()



import pandas as pd
import numpy as np


class BarcodeTitle(object):
    """Bit of a dummy class to add barcode titles to the report"""

    def __init__(self, title):
        self.title = title.upper()

    def encode(self):
        return ""


def chunks(values, chunks):
    if values:
        chunksize = int(len(values) / chunks)
        return ([' '.join(values[i:i + chunksize]) for i in range(0, len(values), chunksize)])
    else:
        return [" "] * chunks


def html_stats(settings):
    statsfile = settings["statsfile"]
    filtered = settings["filtered"]
    as_tsv = settings['tsv_stats']

    stats_html = []
    stats_html.append('<main class="grid-main"><h2>NanoPlot reports</h2>')
    if filtered:
        stats_html.append('<h3 id="stats0">Summary statistics prior to filtering</h3>')
        if as_tsv:
            stats_html.append(statsfile[0].to_html())
            stats_html.append('<h3 id="stats1">Summary statistics after filtering</h3>')
            stats_html.append(statsfile[1].to_html())
        else:
            stats_html.append(stats2html(statsfile[0]))
            stats_html.append('<h3 id="stats1">Summary statistics after filtering</h3>')
            stats_html.append(stats2html(statsfile[1]))
    else:
        stats_html.append('<h3 id="stats0">Summary statistics</h3>')
        if as_tsv:
            stats_html.append(statsfile[0].to_html())
        else:
            stats_html.append(stats2html(statsfile[0]))
    return '\n'.join(stats_html)


def stats2html(statsf):
    df = pd.read_csv(statsf, sep=':', header=None, names=['feature', 'value'])
    values = df["value"].str.strip().str.replace('\t', ' ').str.split().replace(np.nan, '')
    num = len(values[0]) or 1
    v = [chunks(i, num) for i in values]
    df = pd.DataFrame(v, index=df["feature"])
    df.columns.name = None
    df.index.name = None
    return df.to_html(header=False)


def html_toc(plots, filtered=False):
    toc = []
    toc.append('<h1 class="hiddentitle">NanoPlot statistics report</h1>')
    toc.append('<header class="grid-header"><nav><h2 class="hiddentitle">Menu</h2><ul>')
    if filtered:
        toc.append(
            '<li><a href="#stats0">Summary Statistics prior to filtering</a></li>')
        toc.append(
            '<li><a href="#stats1">Summary Statistics after filtering</a></li>')
    else:
        toc.append('<li><a href="#stats0">Summary Statistics</a></li>')

    toc.append('<li class="submenu"><a href="#plots" class="submenubtn">Plots</a>')
    toc.append('<ul class="submenu-items">')
    toc.extend(['<li><a href="#'
                + p.title.replace(' ', '_') + '">' + p.title + '</a></li>' for p in plots])
    toc.append('</ul>')
    toc.append('</li>')
    toc.append(
        '<li class="issue-btn"><a href="https://github.com/wdecoster/NanoPlot/issues" target="_blank"  class="reporting">Report issue on Github</a></li>')
    toc.append('</ul></nav></header>')
    return '\n'.join(toc)


def html_plots(plots):
    html_plots = []
    html_plots.append('<h3 id="plots">Plots</h3>')
    for plot in plots:
        html_plots.append('<button class="collapsible">' + plot.title + '</button>')
        html_plots.append('<section class="collapsible-content"><h4 class="hiddentitle" id="' +
                          plot.title.replace(' ', '_') + '">' + plot.title + '</h4>')
        html_plots.append(plot.encode())
        html_plots.append('</section>')

    html_plots.append(
        '<script>var coll = document.getElementsByClassName("collapsible");var i;for (i = 0; i < coll.length; i++) {coll[i].addEventListener("click", function() {this.classList.toggle("active");var content = this.nextElementSibling;if (content.style.display === "none") {content.style.display = "block";} else {content.style.display = "none";}});}</script>')

    return '\n'.join(html_plots)


def run_info(settings):
    html_info = []
    html_info.append('<h5>Run Info</h5>\n')
    html_info.append('<h6>Data source:</h6>\n')
    for k in ["fastq", "fasta", "fastq_rich", "fastq_minimal", "summary",
              "bam", "ubam", "cram", "pickle", "feather"]:
        html_info.append(f"{k}:\t{settings[k]}<br>")
    html_info.append('<h6>Filtering parameters:</h6>\n')
    for k in ['maxlength', 'minlength', 'drop_outliers', 'downsample', 'loglength',
              'percentqual', 'alength', 'minqual', 'runtime_until', 'no_supplementary']:
        html_info.append(f"{k}:\t{settings[k]}<br>")
    # html_info.append('</p>')
    return '\n'.join(html_info)


html_head = """
<!DOCTYPE html>
<html>
<head>
<meta charset="UTF-8">
<style>

body {margin:0}

.grid { /* grid definition for index page */
    display: grid;
    grid-template-areas:    'gheader'
                            'gmain';
    margin: 0;
}

.grid > .grid-header { /* definition of the header on index page and its position in the grid */
    grid-area: gheader;
}

.grid > .grid-main { /* definition of the main content on index page and its position in the grid */
    grid-area: gmain;
}

nav {
    text-align: center;
}

ul {
    border-bottom: 1px solid white;
    font-family: "Trebuchet MS", sans-serif;
    list-style-type: none; /* remove dot symbols from list */
    margin: 0;
    padding: 0;
    overflow: hidden; /* contains the overflow of the element if it goes 'out of bounds' */
    background-color: #001f3f;
    font-size: 1.6em;
}

ul > li > ul {
    font-size: 1em;
}

li {
    float: left; /* floats the list items to the left side of the page */
}

li a, .submenubutton {
    display: inline-block; /* display the list items inline block so the items are vertically displayed */
    color: white;
    text-align: center;
    padding: 14px 16px;
    text-decoration: none; /* removes the underline that comes with the a tag */
}

li a:hover, .submenu:hover .submenubutton { /* when you hover over a submenu item the bkgrnd color is gray */
    background-color: #39CCCC;
}

.submenu {
    display: inline-block; /* idem to above, list items are displayed underneath each other */
}

.submenu-items { /* hides the ul */
    display: none;
    position: absolute;
    background-color: #f9f9f9;
    min-width: 160px;
    z-index: 1;
}

.submenu-items li {
    display: block;
    float: none;
    overflow: hidden;
}

.submenu-items li a { /* styling of the links in the submenu */
    color: black;
    padding: 12px 16px;
    text-decoration: none;
    display: block;
    text-align: left;
}

.submenu-items a:hover {
    background-color: #f1f1f1;
}

.submenu:hover .submenu-items {
    display: block;
    float: bottom;
    overflow: hidden;
}

li {
  border-right: 1px solid #bbb;
}

.issue-btn {
  border-right: none;
  float: right;
}

.hiddentitle { /* hides titles that are not necessary for content, but are for outline */
  position: absolute;
  width: 1px;
  height: 1px;
  overflow: hidden;
  left: -10000px;
}

h2 { color: #111; font-family: 'Helvetica Neue', sans-serif; font-size: 60px; font-weight: bold; letter-spacing: -1px; line-height: 1; text-align: center; }

h3 { color: #111; font-family: 'Open Sans', sans-serif; font-size: 25px; font-weight: 300; line-height: 32px; text-align: center; padding-bottom: 0;}

h4 { color: #111; font-family: 'Helvetica Neue', sans-serif; font-size: 16px; font-weight: 150; margin: 0 0 0 0; text-align: left; padding:20px 0px 20px 0px;}

table {
  font-family: Arial, Helvetica, sans-serif;
  border-collapse: collapse;
  table-layout: auto;
  border-collapse: collapse;
  width: 100%;
}

table td, table th {
  border: 1px solid #ddd;
  padding: 8px;
}

table tr:nth-child(even){background-color: #f2f2f2;}

table tr:hover {background-color: #ddd;}

/* Style the button that is used to open and close the collapsible content */
.collapsible {
  background-color: #39CCCC;
  color: white;
  cursor: pointer;
  padding: 18px;
  width: 100%;
  border: none;
  text-align: left;
  outline: none;
  font-size: 15px;
}

/* Add a background color to the button if it is clicked on (add the .active class with JS), and when you move the mouse over it (hover) */
.active, .collapsible:hover {
    color:white;
  background-color: #001f3f;
}

/* Style the collapsible content. Note: hidden by default */
.collapsible-content {
  padding: 0 18px;
  display: block;
  overflow: hidden;
  background-color: #FFFFFF;
  text-align: center;
}

.collapsible:after {
  content: '-';
  font-size: 20px;
    font-weight: bold;
  float: right;
    color:white;
  margin-left: 5px;
}

.active:after {
  content: '+'; /* Unicode character for "minus" sign (-) */
      color: white;
}
</style>
<title>NanoPlot Report</title>
</head>
"""



import sys
import os
from datetime import datetime as dt
from time import time
import logging
from nanoplot.version import __version__
from argparse import HelpFormatter, Action, ArgumentParser
import textwrap as _textwrap
import pandas as pd


class CustomHelpFormatter(HelpFormatter):
    def _format_action_invocation(self, action):
        if not action.option_strings or action.nargs == 0:
            return super()._format_action_invocation(action)
        default = self._get_default_metavar_for_optional(action)
        args_string = self._format_args(action, default)
        return ', '.join(action.option_strings) + ' ' + args_string

    def _fill_text(self, text, width, indent):
        return ''.join(indent + line for line in text.splitlines(keepends=True))

    def _split_lines(self, text, width):
        text = self._whitespace_matcher.sub(' ', text).strip()
        return _textwrap.wrap(text, 80)


class Action_Print_Colors(Action):
    def __init__(self, option_strings, dest="==SUPPRESS==", default="==SUPPRESS==", help=None):
        super(Action_Print_Colors, self).__init__(
            option_strings=option_strings,
            dest=dest,
            default=default,
            nargs=0,
            help=help)

    def __call__(self, parser, namespace, values, option_string=None):
        list_colors()


class Action_Print_Colormaps(Action):
    def __init__(self, option_strings, dest="==SUPPRESS==", default="==SUPPRESS==", help=None):
        super(Action_Print_Colormaps, self).__init__(
            option_strings=option_strings,
            dest=dest,
            default=default,
            nargs=0,
            help=help)

    def __call__(self, parser, namespace, values, option_string=None):
        list_colormaps()


def get_args():
    epilog = """EXAMPLES:
    NanoPlot --summary sequencing_summary.txt --loglength -o summary-plots-log-transformed
    NanoPlot -t 2 --fastq reads1.fastq.gz reads2.fastq.gz --maxlength 40000 --plots hex dot
    NanoPlot --color yellow --bam alignment1.bam alignment2.bam alignment3.bam --downsample 10000
    """
    parser = ArgumentParser(
        description="Creates various plots for long read sequencing data.".upper(),
        epilog=epilog,
        formatter_class=custom_formatter,
        add_help=False)
    general = parser.add_argument_group(
        title='General options')
    general.add_argument("-h", "--help",
                         action="help",
                         help="show the help and exit")
    general.add_argument("-v", "--version",
                         help="Print version and exit.",
                         action="version",
                         version='NanoPlot {}'.format(__version__))
    general.add_argument("-t", "--threads",
                         help="Set the allowed number of threads to be used by the script",
                         default=4,
                         type=int)
    general.add_argument("--verbose",
                         help="Write log messages also to terminal.",
                         action="store_true")
    general.add_argument("--store",
                         help="Store the extracted data in a pickle file for future plotting.",
                         action="store_true")
    general.add_argument("--raw",
                         help="Store the extracted data in tab separated file.",
                         action="store_true")
    general.add_argument("--huge",
                         help="Input data is one very large file.",
                         action="store_true")
    general.add_argument("-o", "--outdir",
                         help="Specify directory in which output has to be created.",
                         default=".")
    general.add_argument("-p", "--prefix",
                         help="Specify an optional prefix to be used for the output files.",
                         default="",
                         type=str)
    general.add_argument("--tsv_stats",
                         help="Output the stats file as a properly formatted TSV.",
                         action='store_true')
    general.add_argument("--info_in_report",
                         help="Add NanoPlot run info in the report.",
                         action='store_true')
    filtering = parser.add_argument_group(
        title='Options for filtering or transforming input prior to plotting')
    filtering.add_argument("--maxlength",
                           help="Hide reads longer than length specified.",
                           type=int,
                           metavar='N')
    filtering.add_argument("--minlength",
                           help="Hide reads shorter than length specified.",
                           type=int,
                           metavar='N')
    filtering.add_argument("--drop_outliers",
                           help="Drop outlier reads with extreme long length.",
                           action="store_true")
    filtering.add_argument("--downsample",
                           help="Reduce dataset to N reads by random sampling.",
                           type=int,
                           metavar='N')
    filtering.add_argument("--loglength",
                           help="Additionally show logarithmic scaling of lengths in plots.",
                           action="store_true")
    filtering.add_argument("--percentqual",
                           help="Use qualities as theoretical percent identities.",
                           action="store_true")
    filtering.add_argument("--alength",
                           help="Use aligned read lengths rather than sequenced length (bam mode)",
                           action="store_true")
    filtering.add_argument("--minqual",
                           help="Drop reads with an average quality lower than specified.",
                           type=int,
                           metavar='N')
    filtering.add_argument("--runtime_until",
                           help="Only take the N first hours of a run",
                           type=int,
                           metavar='N')
    filtering.add_argument("--readtype",
                           help="Which read type to extract information about from summary. \
                                 Options are 1D, 2D, 1D2",
                           default="1D",
                           choices=['1D', '2D', '1D2'])
    filtering.add_argument("--barcoded",
                           help="Use if you want to split the summary file by barcode",
                           action="store_true")
    filtering.add_argument("--no_supplementary",
                           help="Use if you want to remove supplementary alignments",
                           action="store_true",
                           default=False)
    visual = parser.add_argument_group(
        title='Options for customizing the plots created')
    visual.add_argument("-c", "--color",
                        help="Specify a valid matplotlib color for the plots",
                        default="#4CB391")
    visual.add_argument("-cm", "--colormap",
                        help="Specify a valid matplotlib colormap for the heatmap",
                        default="Greens")
    visual.add_argument("-f", "--format",
                        help="Specify the output format of the plots, which are in addition to the html files",
                        default="png",
                        type=str,
                        choices=['png','jpg','jpeg','webp','svg','pdf','eps','json'])
    visual.add_argument("--plots",
                        help="Specify which bivariate plots have to be made.",
                        default=['kde', 'dot'],
                        type=str,
                        nargs='*',
                        choices=['kde', 'hex', 'dot'])
    visual.add_argument("--legacy", help="Specify which bivariate plots have to be made (legacy mode).",
                        type=str,
                        nargs='*',
                        choices=['kde', 'dot', 'hex'])
    visual.add_argument("--listcolors",
                        help="List the colors which are available for plotting and exit.",
                        action=Action_Print_Colors,
                        default=False)
    visual.add_argument("--listcolormaps",
                        help="List the colors which are available for plotting and exit.",
                        action=Action_Print_Colormaps,
                        default=False)
    visual.add_argument("--no-N50",
                        help="Hide the N50 mark in the read length histogram",
                        action="store_true")
    visual.add_argument("--N50",
                        help="Show the N50 mark in the read length histogram",
                        action="store_true",
                        default=False)
    visual.add_argument("--title",
                        help="Add a title to all plots, requires quoting if using spaces",
                        type=str,
                        default=None)
    visual.add_argument("--font_scale",
                        help="Scale the font of the plots by a factor",
                        type=float,
                        default=1)
    visual.add_argument("--dpi",
                        help="Set the dpi for saving images",
                        type=int,
                        default=100)
    visual.add_argument("--hide_stats",
                        help="Not adding Pearson R stats in some bivariate plots",
                        action="store_true",
                        default=False)
    target = parser.add_argument_group(
        title="Input data sources, one of these is required.")
    mtarget = target.add_mutually_exclusive_group(
        required=True)
    mtarget.add_argument("--fastq",
                         help="Data is in one or more default fastq file(s).",
                         nargs='+',
                         metavar="file")
    mtarget.add_argument("--fasta",
                         help="Data is in one or more fasta file(s).",
                         nargs='+',
                         metavar="file")
    mtarget.add_argument("--fastq_rich",
                         help="Data is in one or more fastq file(s) generated by albacore, \
                               MinKNOW or guppy with additional information \
                               concerning channel and time.",
                         nargs='+',
                         metavar="file")
    mtarget.add_argument("--fastq_minimal",
                         help="Data is in one or more fastq file(s) generated by albacore, \
                               MinKNOW or guppy with additional information concerning channel \
                               and time. Is extracted swiftly without elaborate checks.",
                         nargs='+',
                         metavar="file")
    mtarget.add_argument("--summary",
                         help="Data is in one or more summary file(s) generated by albacore \
                               or guppy.",
                         nargs='+',
                         metavar="file")
    mtarget.add_argument("--bam",
                         help="Data is in one or more sorted bam file(s).",
                         nargs='+',
                         metavar="file")
    mtarget.add_argument("--ubam",
                         help="Data is in one or more unmapped bam file(s).",
                         nargs='+',
                         metavar="file")
    mtarget.add_argument("--cram",
                         help="Data is in one or more sorted cram file(s).",
                         nargs='+',
                         metavar="file")
    mtarget.add_argument("--pickle",
                         help="Data is a pickle file stored earlier.",
                         metavar="pickle")
    mtarget.add_argument("--feather",
                         help="Data is in one or more feather file(s).",
                         nargs='+',
                         metavar="file")
    args = parser.parse_args()
    if args.listcolors:
        list_colors()
    if args.listcolormaps:
        list_colormaps()
    if args.no_N50:
        sys.stderr.write('DeprecationWarning: --no-N50 is currently the default setting.\n')
        sys.stderr.write('The argument is thus unnecessary but kept for backwards compatibility.')
    if args.barcoded and not args.summary:
        sys.exit('ARGUMENT ERROR: --barcoded only works with data provided as --summary!')
    settings = vars(args)
    settings["path"] = os.path.join(args.outdir, args.prefix)
    return settings, args


def custom_formatter(prog):
    return CustomHelpFormatter(prog)


def list_colors():
    parent_directory = os.path.dirname(os.path.abspath(os.path.dirname(__file__)))
    colours = open(os.path.join(parent_directory, "extra/color_options_hex.txt"))
    col_hex = {}

    for line in colours:
        key, value = line.split(",")
        col_hex[key] = value.strip()
    print("Valid colors: {}".format("\n".join([c.strip() for c in list(col_hex.keys())])))
    sys.exit(0)


def list_colormaps():
    print('Valid colormaps:\nGreys\nYlGnBu\nGreens\nYlOrRd\nBluered\nRdBu\nReds\nBlues\nPicnic\n'
          'Rainbow\nPortland\nJet\nHot\nBlackbody\nEarth\nElectric\nViridis\nCividis')
    sys.exit(0)


def make_output_dir(path):
    try:
        if not os.path.exists(path):
            os.makedirs(path)
    except IOError:
        sys.exit("ERROR: No writing permission to the output directory.")


def init_logs(args, tool="NanoPlot"):
    """Initiate log file and log arguments."""
    start_time = dt.fromtimestamp(time()).strftime('%Y%m%d_%H%M')
    logname = os.path.join(args.outdir, args.prefix + tool + "_" + start_time + ".log")
    handlers = [logging.FileHandler(logname)]
    if args.verbose:
        handlers.append(logging.StreamHandler())
    logging.basicConfig(
        format='%(asctime)s %(message)s',
        handlers=handlers,
        level=logging.INFO)
    logging.info('{} {} started with arguments {}'.format(tool, __version__, args))
    logging.info('Python version is: {}'.format(sys.version.replace('\n', ' ')))
    return logname


def subsample_datasets(df, minimal=10000):
    if 'dataset' in df:
        list_df = []

        for d in df["dataset"].unique():
            dataset = df.loc[df['dataset'] == d]

            if len(dataset.index) < minimal:
                list_df.append(dataset)

            else:
                list_df.append(dataset.sample(minimal))

        subsampled_df = pd.concat(list_df, ignore_index=True)

    else:
        if len(df.index) < minimal:
            subsampled_df = df

        else:
            subsampled_df = df.sample(minimal)

    return subsampled_df
